import jinja2
from os.path import expanduser, dirname, join
import subprocess
import json
from squad_report.reports import squad_build_url
from . import squad_get
import squad_report.logging

logger = squad_report.logging.getLogger(__name__)

te = jinja2.Environment(
    extensions=["jinja2.ext.loopcontrols"],
    loader=jinja2.ChoiceLoader(
        [
            jinja2.FileSystemLoader(
                join(dirname(__file__), "..", "squad_report", "templates")
            ),
            jinja2.FileSystemLoader(expanduser("~/.config/squad_report/templates")),
        ]
    ),
    trim_blocks=True,
    lstrip_blocks=True,
    undefined=jinja2.StrictUndefined,
)


def create_perf_report(
    config, perf_report_args, group, project, build, base_build, environments, suites
):
    mmtests_directory = perf_report_args[0]

    path_to_mmtests = join(dirname(__file__), "../../", mmtests_directory)
    path_to_compare = join(path_to_mmtests, "compare-kernels.sh")

    environment = environments.split(",")[0]
    suite = suites.split(",")[0]
    testname = suite.split("-")[1]

    dict_data = squad_get.get_data(base_build, build, environment, suite)
    json_files = []

    for key in dict_data:
        json_filename = key + ".json"
        with open(json_filename, "w") as fp:
            json_files.append(json_filename)
            json.dump(dict_data[key], fp)

    cmp_kern_out = b""

    for jsonfile in json_files:
        cmd_compare_kernel = [
            f"{path_to_compare}",
            "--from-json",
            jsonfile,
        ]

        cmp_kern_out += subprocess.check_output(cmd_compare_kernel)

    full_output = []
    regression = []
    for line in cmp_kern_out.splitlines():
        full_output.append(line.decode())
        if "*" in line.decode():
            regression.append(line.decode())

    doc_link = (
        "https://github.com/gormanm/mmtests/blob/master/docs/regression-detection.md"
    )

    config_values = {"item": [], "test": "test=" + testname, "cmd": ""}
    seen_cmds = []

    try:
        for jsonfile in json_files:
            benchmark = json.load(open(jsonfile, "r"))
            for i in range(len(benchmark)):
                if "_Cmd" in benchmark[i]:
                    cmd_string = benchmark[i]["_Cmd"]
                    if cmd_string not in seen_cmds:
                        seen_cmds.append(cmd_string)
                        config_values["cmd"] += "\n" + cmd_string
                        break

            config_found = False
            for i in range(len(benchmark)):
                if config_found:
                    break
                for key in benchmark[i]:
                    if "_CONFIG-" in key:
                        config_values["item"].append(
                            key.replace("_", "") + "=" + benchmark[i][key]
                        )
                        config_found = True
    except OSError:
        logger.error("unable to open json file")

    args = {
        "build": build,
        "build_url": squad_build_url(group, project, build),
        "base_build": base_build,
        "config": config,
        "doc": doc_link,
        "compare_kernel": {"info": config_values},
        "regression_output": regression,
        "complete_output": full_output,
    }
    return te.get_template(config.template + ".txt.jinja").render(**args)



from base64 import b64decode
import squad_report.logging
from squad_client.core.api import SquadApi

logger = squad_report.logging.getLogger(__name__)


def get_single_test_data(build, environment, suite):
    SquadApi.configure(url="https://qa-reports.linaro.org/")

    filters = {"environment__slug": environment, "metadata__suite": suite}

    build_metrics = {m.name: m.result for m in build.metrics(**filters).values()}

    if not build_metrics:
        raise ValueError(f"Data not found for {suite} on {environment}")

    result = {}
    first_module_name = None
    module_name = None
    operation = None
    cmd_pieces = {}

    for key, val in build_metrics.items():
        job_data = key.split("/")[1].split("_")

        if not first_module_name:
            first_module_name = job_data[0]
        module_name = job_data[0]

        if module_name not in result:
            result[module_name] = {
                "_ResultData": {},
                "_TestName": module_name,
                "_ModuleName": "Extract" + module_name.capitalize(),
                "_Cmd": "",
            }

        operation = str(job_data[1])
        iteration = int(job_data[2])
        sample = str(job_data[3])

        if operation == "#Cmd":
            if module_name not in cmd_pieces:
                cmd_pieces[module_name] = [{"pos": iteration, "text": sample}]
            else:
                cmd_pieces[module_name].append({"pos": iteration, "text": sample})
        elif "#CONFIG-" in operation:
            result[module_name][operation.replace("#", "_")] = sample
        else:
            if operation not in result[module_name]["_ResultData"]:
                result[module_name]["_ResultData"][operation] = []

                operation_length = len(result[module_name]["_ResultData"][operation])
                if operation_length <= iteration:
                    appends_to_do = (iteration + 1) - operation_length

                    for i in range(appends_to_do):
                        result[module_name]["_ResultData"][operation].append(
                            {"SampleNrs": [], "Values": []}
                        )

            result[module_name]["_ResultData"][operation][iteration][
                "SampleNrs"
            ].append(sample)
            result[module_name]["_ResultData"][operation][iteration]["Values"].append(
                val
            )

    for key, pieces in cmd_pieces.items():
        pieces.sort(key=lambda x: x["pos"])

        b64string = ""
        counter = -1
        for piece in pieces:
            if piece["pos"] > counter:
                counter = piece["pos"]
                b64string += piece["text"]

        # cmd string comes with no padding, add if needed
        missing_padding = len(b64string) % 4
        if missing_padding:
            b64string += "=" * (4 - missing_padding)

        try:
            decoded_string = b64decode(b64string).decode("utf-8")
        except Exception as e:
            logger.error(e)
            decoded_string = ""

        result[key]["_Cmd"] = decoded_string

    return result


def get_data(base_build, build, environment, suite):

    dict_data = {}

    try:
        data_base = get_single_test_data(base_build, environment, suite)
        data_current = get_single_test_data(build, environment, suite)
    except ValueError as e:
        logger.error(e)
        return

    for key in data_base:
        item = [data_base[key], data_current[key]]
        item[0]["_TestName"] = getattr(base_build, "version")
        item[1]["_TestName"] = getattr(build, "version")
        dict_data[key] = item

    return dict_data
